/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.util.EnumUtil.StructureChoice;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is structures endpoint and device group.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
public class StructuresDeviceGroupIT {

    // note
    //     StructureElement - when, who, comment - depend on requested or processed
    //         requested, requested by, requested comment
    //         processed, processed by, processed comment
    //     if less than a second between requested and processed, then considered one entry with processed
    //     history
    //         mnemonic path does not make same sense for history
    //         (very) tricky to find mnemonic path for uuid at proper time (history)
    //         therefore empty mnemonic path for history for structure
    //         one history entry if less than one second between requested and processed, otherwise two history entries
    //     attributes for entry for operations - create, update, delete, approve, cancel, reject
    //         some set client side, others set server side
    //         client side
    //             type, uuid, parent uuid, name, mnemonic, description, comment
    //         may be set client side for test purposes
    //     order of methods in test not known
    //         therefore mnemonic values chosen to not interfere with mnemonic values in other methods
    //
    //     StructureElement extends StructureElementCommand
    //     -->
    //     for such operations that send Command objects to backend, it is also possible to send StructureElement

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-it-db-schema-migration.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID disciplineUuid  = null;
    private static UUID discipline2Uuid = null;

    @BeforeAll
    public static void initAll() {
        // init discipline

        StructureElementCommand structureElement  = null;
        StructureElement approvedStructureElement = null;

        structureElement = new StructureElementCommand(
                null, Type.DISCIPLINE, null,
                "name", "Di",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        disciplineUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.DISCIPLINE, null,
                "name", "Di2",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        discipline2Uuid = approvedStructureElement.getUuid();
    }

    @Test
    public void equivalenceMnemonic() {
        // purpose
        //     test mnemonic equivalence
        //
        // what
        //     read        equivalence mnemonic

        ITUtilStructureElement.assertEquivalence("Dg", "DG");
    }

    @Test
    public void checkCreate() {
        // purpose
        //     test conditions for create device group
        //         not create itself
        //
        // what - combination of
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //
        // validate create
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            StructureElementCommand structureElement = new StructureElementCommand();

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "CC", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "CC", Boolean.FALSE);

            ITUtilStructureElement.assertValidate("[{asdf]", StructureChoice.CREATE, HttpURLConnection.HTTP_BAD_REQUEST);
            ITUtilStructureElement.assertCreate("[{asdf]", HttpURLConnection.HTTP_BAD_REQUEST);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setParent(disciplineUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setType(Type.DEVICEGROUP);
            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkCreateMnemonic() {
        // purpose
        //     test conditions for create device group
        //         not create itself
        //
        // what - combination of
        //     read        validate create
        //
        // note
        //     mnemonic
        try {
            StructureElementCommand structureElement = new StructureElementCommand();

            structureElement.setType(Type.DEVICEGROUP);
            structureElement.setParent(disciplineUuid);
            structureElement.setName("name");
            structureElement.setDescription("description");
            structureElement.setComment("comment");

            // mnemonic rules

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("C");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Ccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Cccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Ccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Cccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Ccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Cccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Ccccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            // mnemonic rules (2)

            structureElement.setMnemonic(" ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Grp ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Grp");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("000");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Grp0");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic(":");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Grp:");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Grp:   ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("1");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("12");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("123");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("1234");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("12345");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("123456");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("1234567");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("12345678");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("123456789");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createApprove() {
        // purpose
        //     test create and approve system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create in order to approve

        try {
            StructureElementCommand structureElement = null;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // create
            ITUtilStructureElement.assertCreate(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createCancel() {
        // purpose
        //     test create and cancel system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate cancel
        //     patch       cancel structures
        //
        // note
        //     create in order to cancel

        try {
            StructureElementCommand structureElement = null;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // create
            ITUtilStructureElement.assertCreate(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            ITUtilStructureElement.assertCancel(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createReject() {
        // purpose
        //     test create and reject system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate reject
        //     patch       reject structures
        //
        // note
        //     create in order to reject

        try {
            StructureElementCommand structureElement = null;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // create
            ITUtilStructureElement.assertCreate(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            ITUtilStructureElement.assertReject(structureElement);

            ITUtilStructureElement.assertExists         (Type.DEVICEGROUP, "Di", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkUpdate() {
        // purpose
        //     test conditions for update device group
        //         not update
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update
        //
        // validate update
        //     uuid
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            UUID uuid = null;

            StructureElementCommand structureElement  = new StructureElementCommand();
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            uuid = approvedStructureElement.getUuid();

            // validate update

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setParent(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setParent(disciplineUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setMnemonic("Cu");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateApprove() {
        // purpose
        //     test update and approve device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     update      update structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update, approve

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment update approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateCancel() {
        // purpose
        //     test update and cancel device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate cancel
        //     update      update structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to update, cancel

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment update cancel check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            ITUtilStructureElement.assertCancel(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateReject() {
        // purpose
        //     test update and reject device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate reject
        //     update      update structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to update, reject

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment update reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            ITUtilStructureElement.assertReject(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkDelete() {
        // purpose
        //     test conditions for delete device group
        //         not delete
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete
        //
        // validate delete
        //     uuid
        //     type
        //     comment

        try {
            UUID uuid = null;

            StructureElementCommand structureElement  = new StructureElementCommand();
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            uuid = approvedStructureElement.getUuid();

            // validate delete

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setParent(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setParent(disciplineUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setMnemonic("Cd");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteApprove() {
        // purpose
        //     test delete and approve device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     delete      delete structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete, approve

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment delete approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteCancel() {
        // purpose
        //     test delete and cancel device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate cancel
        //     update      delete structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to delete, cancel

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment update delete check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            ITUtilStructureElement.assertCancel(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteReject() {
        // purpose
        //     test delete and reject device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate reject
        //     update      delete structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to delete, reject

        try {
            StructureElementCommand structureElement = null;

            // create, approve
            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            ITUtilStructureElement.assertCreateApprove(structureElement);

            structureElement.setComment("comment delete reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            ITUtilStructureElement.assertReject(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryApprove() {
        // purpose
        //     test read device group in various ways for create, approve
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommand structureElement  = null;
            StructureElement createdStructureElement  = null;
            StructureElement approvedStructureElement = null;
            int length = 0;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rsha",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rsha",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1));

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + approvedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rsha",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rsha",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + approvedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryCancel() {
        // purpose
        //     test read device group in various ways for create, cancel
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommand structureElement   = null;
            StructureElement createdStructureElement   = null;
            StructureElement cancelledStructureElement = null;
            int length = 0;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshc",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rshc",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1));

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + cancelledStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshc",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rshc",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + cancelledStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryReject() {
        // purpose
        //     test read device group in various ways for create, reject
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommand structureElement  = null;
            StructureElement createdStructureElement  = null;
            StructureElement rejectedStructureElement = null;
            int length = 0;

            structureElement = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshr",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rshr",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", -1, 1));

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  -1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + rejectedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",         0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshr",      0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Di",      -1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr",     0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Di-Rshr",  0, 0);

            ITUtilStructureElement.assertRead("/history/" + rejectedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest1() {
        // purpose
        //     test read device type in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         1 entry for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElementCommand structureElement  = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        uuid = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        uuid2 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        uuid3 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/C1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/R1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/A1", 0, 0);

            // normally expected number of entries are 1, 2, 2, 2
            // since within 1 second from requested to processed, expected number of entries 1, 1, 1, 1
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-P1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-C1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-R1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-A1", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest9() {
        // purpose
        //     test read device type in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         9 entries for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElementCommand structureElement  = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        responseStructureElement.setComment("comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdate(responseStructureElement);

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid2 = responseStructureElement.getUuid();

        responseStructureElement.setComment("comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid3 = responseStructureElement.getUuid();

        responseStructureElement.setComment("comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        responseStructureElement.setComment("comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid.toString(),  2, 2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid2.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid3.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P9", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/C9", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/R9", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/A9", 0, 0);

            // normally expected number of entries are 17, 18, 18, 18
            // since within 1 second from requested to processed, expected number of entries 9, 9, 9, 9
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 9, 9);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-P9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-C9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-R9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-A9", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchStatusDeletedChildren() {
        // purpose
        //     test read device group in various ways
        //         status
        //       ( latest )
        //         deleted
        //         children
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     querying for Status.APPROVED means latest approved

        StructureElementCommand structureElement  = null;
        StructureElement responseStructureElement = null;
        UUID uuid = null;

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        String description2 = "some other description";
        String comment2 = "some other comment";
        String description3 = "more description";
        String comment3 = "more comment";

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElementCommand(
                null, Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        // 60 device group entries

        try {
            // from first structure element
            assertNotNull(uuid);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?queryFields=MNEMONICPATH&queryValues=Di2",                                                  30, 30);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&queryFields=MNEMONICPATH&queryValues=Di2",                                  5,  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2",                                20, 20);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&queryFields=MNEMONICPATH&queryValues=Di2",                                0,  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&queryFields=MNEMONICPATH&queryValues=Di2",                                 5,  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2",               25, 25);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&queryFields=MNEMONICPATH&queryValues=Di2",                                    15, 15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=PENDING&queryFields=MNEMONICPATH&queryValues=Di2",                    0,  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2",                  15, 15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=CANCELLED&queryFields=MNEMONICPATH&queryValues=Di2",                  0,  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=REJECTED&queryFields=MNEMONICPATH&queryValues=Di2",                   0,  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2", 15, 15);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&queryFields=MNEMONICPATH&queryValues=Di2",                                     15, 15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=PENDING&queryFields=MNEMONICPATH&queryValues=Di2",                     5,  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2",                    5,  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=CANCELLED&queryFields=MNEMONICPATH&queryValues=Di2",                   0,  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=REJECTED&queryFields=MNEMONICPATH&queryValues=Di2",                    5,  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONICPATH&queryValues=Di2",  10, 10);

            ITUtilStructureElement.assertRead("/children/DEVICEGROUP/" + uuid.toString(),           0, 0);
            ITUtilStructureElement.assertRead("/children/DISCIPLINE/" + disciplineUuid.toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di",  Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di2", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

}
